/*
 * This file and its contents are supplied under the terms of the
 * Common Development and Distribution License ("CDDL"), version 1.0.
 * You may only use this file in accordance with the terms of version
 * 1.0 of the CDDL.
 *
 * A full copy of the text of the CDDL should have accompanied this
 * source.  A copy of the CDDL is also available via the Internet at
 * http://www.illumos.org/license/CDDL.
 */

/*
 * Copyright 2013 Damian Bogel.  All rights reserved.
 */

#ifndef _FSH_H
#define	_FSH_H

#include <sys/types.h>
#include <sys/vfs.h>
#include <sys/vnode.h>

#ifdef __cplusplus
extern "C" {
#endif

struct fsh_node;
typedef struct fsh_node fsh_node_t;

#define	FSH_OPS	\
int (*hook_open)(const fsh_node_t *fsh_node, void *arg, vnode_t **vpp,	\
	int mode, cred_t *cr, caller_context_t *ct);			\
int (*hook_close)(const fsh_node_t *fsh_node, void *arg, vnode_t *vp,	\
	int flag, int count, offset_t offset, cred_t *cr,		\
	caller_context_t *ct);						\
int (*hook_read)(const fsh_node_t *fsh_node, void *arg, vnode_t *vp,	\
	uio_t *uiop, int ioflag, cred_t *cr, caller_context_t *ct);	\
int (*hook_write)(const fsh_node_t *fsh_node, void *arg, vnode_t *vp,	\
	uio_t *uiop, int ioflag, cred_t *cr, caller_context_t *ct);	\
/* vfs */								\
int (*hook_mount)(const fsh_node_t *fsh_node, void *arg, vfs_t *vfsp,	\
	vnode_t *mvp, struct mounta *uap, cred_t *cr);			\
int (*hook_unmount)(const fsh_node_t *fsh_node, void *arg, vfs_t *vfsp,	\
	int flag, cred_t *cr);						\
int (*hook_root)(const fsh_node_t *fsh_node, void *arg, vfs_t *vfsp,	\
	vnode_t **vpp);							\
int (*hook_statfs)(const fsh_node_t *fsh_node, void *arg, vfs_t *vfsp,	\
	statvfs64_t *sp);						\
int (*hook_vget)(const fsh_node_t *fsh_node, void *arg, vfs_t *vfsp,	\
	vnode_t **vpp, fid_t *fidp) /* NO ';' HERE */

/* API */
typedef struct fsh {
	void	*arg;
	FSH_OPS;
} fsh_t;

extern void fsh_hook_install(vfs_t *vfsp, fsh_t *hooks);
extern void fsh_hook_remove(vfs_t *vfsp, fsh_t *hooks);
extern void fsh_hook_check(vfs_t *vfsp, fsh_t *hooks, fsh_t *mask);

typedef struct fsh_callback {
	void	*fshc_arg;
	void	(*fshc_mount)(vfs_t *vfsp, void *arg);
	void	(*fshc_free)(vfs_t *vfsp, void *arg);
} fsh_callback_t;

extern void fsh_callback_install(fsh_callback_t *fsh_callback);
extern void fsh_callback_remove(fsh_callback_t *fsh_callback);

extern void fsh_fs_enable(vfs_t *vfsp);
extern void fsh_fs_disable(vfs_t *vfsp);

/* FSH control passing */
extern int fsh_next_open(fsh_node_t *fsh_node, vnode_t **vpp,
		int mode, cred_t *cr, caller_context_t *ct);
extern int fsh_next_close(fsh_node_t *fsh_node, vnode_t *vp,
		int flag, int count, offset_t offset, cred_t *cr,
		caller_context_t *ct);
extern int fsh_next_read(fsh_node_t *fsh_node, vnode_t *vp,
		uio_t *uiop, int ioflag, cred_t *cr, caller_context_t *ct);
extern int fsh_next_write(fsh_node_t *fsh_node, vnode_t *vp,
		uio_t *uiop, int ioflag, cred_t *cr, caller_context_t *ct);

extern int fsh_next_mount(fsh_node_t *fsh_node, vfs_t *vfsp,
		vnode_t *mvp, struct mounta *uap, cred_t *cr);
extern int fsh_next_unmount(fsh_node_t *fsh_node, vfs_t *vfsp,
		int flag, cred_t *cr);
extern int fsh_next_root(fsh_node_t *fsh_node, vfs_t *vfsp,
		vnode_t **vpp);
extern int fsh_next_statfs(fsh_node_t *fsh_node, vfs_t *vfsp, statvfs64_t *sp);
extern int fsh_next_vget(fsh_node_t *fsh_node, vfs_t *vfsp,
		vnode_t **vpp, fid_t *fidp);

#ifdef __cplusplus
}
#endif

#endif /* _FSH_H */
